"""
Functions for working with xdg paths and environment.

The :func:`setenv` function sets all XDG single directory environment variables to their values. It is also called on module instanciation.
"""
import os

from torxtools.pathutils import expandpath

__all__ = [
    "setenv",
]

_defaults = {
    "XDG_CONFIG_DIR": "/etc/xdg",
    "XDG_DATA_DIR": "/usr/local/share:/usr/share",
    "XDG_CACHE_HOME": "$HOME/.cache",
    "XDG_CONFIG_HOME": "$HOME/.config",
    "XDG_DATA_HOME": "$HOME/.local/share",
    "XDG_RUNTIME_DIR": None,
}


def setenv():
    """
    XDG Base Directory Specification environment variables setter.

    Sets all XDG environment variables (:ev:`XDG_CACHE_HOME`,
    :ev:`XDG_CONFIG_HOME`, :ev:`XDG_DATA_HOME`, :ev:`XDG_CONFIG_DIRS`,
    :ev:`XDG_DATA_DIRS`, and :ev:`XDG_RUNTIME_DIR`) to their respective values
    according to current os.environ or their defaults.

    All paths will be expanded before being set.

    Example
    -------

    .. code-block:: python

        from torxtools import xdgutils, pathutils

        try:
            del os.environ["XDG_CONFIG_HOME"]
        except:
            pass
        # We changed the XDG environment:
        xdgutils.setenv()
        assert os.environ["XDG_CONFIG_HOME"] == pathutils.expandpath("$HOME/.config")

    See Also
    --------
    `XDG Base Directory Specification <https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html>`__
    """
    for k, v in _defaults.items():
        path = os.environ.get(k, v)
        if path:
            os.environ[k] = expandpath(path)


# call on instanciation
setenv()



"""Torxtools

"""
#
# https://pypi.org/project/torxtools/
#
import sys

__version__ = "0.0.4"

# Used by Makefile to get the version
if sys.argv[1] == "version":
    print(__version__)
    sys.exit(0)

from setuptools import setup, find_packages

__author__ = "Julien Lecomte"
__contact__ = "julien@lecomte.at"
__copyright__ = "2020, Julien Lecomte"
__url__ = "https://gitlab.com/jlecomte/projects/torxtools"
__license__ = "MIT"

# Used setup.py, but not sphinx-build
if sys.argv[0] == "setup.py":
    with open("requirements.txt") as f:
        requirements = f.read().splitlines()

    setup(
        name="torxtools",
        description="Less generic functionalities than Phillips and Pozidriv tools.",
        version=__version__,
        license=__license__,
        author=__author__,
        author_email=__contact__,
        url=__url__,
        long_description=__doc__,
        long_description_content_type="text/plain",
        python_requires=">=3.5",
        packages=find_packages(exclude=["docs*", "tests*"]),
        install_requires=requirements,
        include_package_data=True,
        classifiers=[
            # See: https://pypi.python.org/pypi?:action=list_classifiers
            "Topic :: Utilities",
            "Topic :: Software Development :: Libraries",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            # List of python versions and their support status:
            # https://en.wikipedia.org/wiki/CPython#Version_history
            "Development Status :: 2 - Pre-Alpha",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
        ],
    )



"""
Functions for working with filesystem paths.
"""
import os

import boltons.pathutils

__all__ = [
    "expandpath",
]


def expandpath(path):
    """
    Recursive shell-like expansion of environment variables and tilde home directory.

    Parameters
    ----------
    path: str, [str], None
        a single path, a list of paths, or none.

    Returns
    -------
    str, [str], None:
        a single expanded path, a list of expanded path, or none

    Example
    -------

    .. code-block:: python

        import os
        from torxtools import pathutils
        os.environ['SPAM'] = 'eggs'
        assert pathutils.expandpath(['~/$SPAM/one', '~/$SPAM/two']) == [os.path.expanduser('~/eggs/one'), os.path.expanduser('~/eggs/two')]

    See Also
    --------
    :py:func:`boltons:boltons.pathutils.expandpath`
    """

    def _expandpath(path):
        if path is None:
            return None
        if isinstance(path, list):
            return [_expandpath(p) for p in path]
        return boltons.pathutils.expandpath(path)

    return _expandpath(path)
